MainWindow #translateLine, ScanPopup #translateLine, MainWindow #wordList, MainWindow #dictsPane #dictsList, MainWindow #historyPane #historyList, MainWindow #favoritesPane #favoritesTree {
}


#translateLine[noResults="true"] {
    background: #febb7d;
}

#navToolbar #menuButton::menu-indicator {
    image: none;
}

#translateLine {
    color: palette(Text); /* ensure the text is inverse of the background, and also placehold text is grey */
    margin-left: 0;
    padding: 1px 3px 1px 3px;

    border: 1px solid gray;
    border-left: 0; /* When not in #searchPane the border-left is provided by GroupComboBox */

    border-radius: 3px;
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
}

#searchPane #translateLine {
    border: 1px solid gray;
    border-radius: 0;
}

GroupComboBox {
    margin-right: 0;
    border: 1px solid gray;
    border-radius: 3px;
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
    padding: 2px 4px 2px 4px;

    background: palette(window); /* Distinguish the combobox from the translate line by make it darker*/
    color: palette(Text);
}

#searchPane GroupComboBox{
    border-radius: 3px;
}

GroupComboBox::drop-down {
    subcontrol-origin: padding;
    subcontrol-position: top right;
    width: 15px;
    border-left: 1px solid gray;
    border-right: 1px transparent;

    /* Hack: Padding must be >= 2px, or the drop-down won't show full content */
    padding-left: 2px;
    padding-right: 2px;
}

GroupComboBox::down-arrow {
    image: url(:/icons/1downarrow.svg);
    max-height: 100%;
}


ArticleView #searchText[noResults="true"] {
    background: #febb7d;
}

MainStatusBar #text {
    border-top-right-radius: 3px;
    border-top: 1px solid palette(dark);
    border-right: 1px solid palette(dark);
    background: palette(window);
    padding: 2px;
}

MainStatusBar #icon {
    border-top: 1px solid palette(dark);
    background: palette(window);
    padding: 2px;
    padding-left: 4px;
    padding-right: 0;
}

.ScanPopup #outerFrame {
    border: 1px solid palette(dark);
}

.ScanPopup MainStatusBar #text {
    border-bottom: 1px solid palette(dark);
}

.ScanPopup MainStatusBar[hasImage="false"] #text {
    border-left: 1px solid palette(dark);
}

.ScanPopup MainStatusBar #icon {
    border-left: 1px solid palette(dark);
    border-bottom: 1px solid palette(dark);
}
