#define INITGUID
#include <basetyps.h>

DEFINE_GUID(IID_IUIAutomation,            0x30cbe57d, 0xd9d0, 0x452a, 0xab, 0x13, 0x7a, 0xc5, 0xac, 0x48, 0x25, 0xee);
DEFINE_GUID(CLSID_CUIAutomation,          0xff48dba4, 0x60ef, 0x4201, 0xaa, 0x87, 0x54, 0x10, 0x3e, 0xef, 0x59, 0x4e);
DEFINE_GUID(IID_IUIAutomationElement,     0xd22108aa, 0x8ac5, 0x49a5, 0x83, 0x7b, 0x37, 0xbb, 0xb3, 0xd7, 0x59, 0x1e);
DEFINE_GUID(IID_IUIAutomationTextPattern, 0x32eba289, 0x3583, 0x42c9, 0x9c, 0x59, 0x3b, 0x6d, 0x9a, 0x1e, 0x9b, 0x6a);
DEFINE_GUID(IID_IUIAutomationTextRange,   0xa543cc6a, 0xf4ae, 0x494b, 0x82, 0x39, 0xc8, 0x14, 0x48, 0x11, 0x87, 0xa8);
DEFINE_GUID(IID_IUIAutomationTreeWalker,  0x4042c624, 0x389c, 0x4afc, 0xa6, 0x30, 0x9d, 0xf8, 0x54, 0xa5, 0x41, 0xfc);

DEFINE_GUID(CLSID_SpVoice,                0x96749377, 0x3391, 0x11D2, 0x9e, 0xe3, 0x00, 0xc0, 0x4f, 0x79, 0x73, 0x96);
DEFINE_GUID(IID_ISpVoice,                 0x6C44DF74, 0x72B9, 0x4992, 0xa1, 0xec, 0xef, 0x99, 0x6e, 0x04, 0x22, 0xd4);
DEFINE_GUID(CLSID_SpObjectToken,          0xEF411752, 0x3736, 0x4CB4, 0x9c, 0x8c, 0x8e, 0xf4, 0xcc, 0xb5, 0x8e, 0xfe);
DEFINE_GUID(IID_ISpObjectToken,           0x14056589, 0xE16C, 0x11D2, 0xbb, 0x90, 0x00, 0xc0, 0x4f, 0x8e, 0xe6, 0xc0);
DEFINE_GUID(CLSID_SpObjectTokenCategory,  0xA910187F, 0x0C7A, 0x45AC, 0x92, 0xcc, 0x59, 0xed, 0xaf, 0xb7, 0x7b, 0x53);
DEFINE_GUID(IID_ISpObjectTokenCategory,   0x2D3D3845, 0x39AF, 0x4850, 0xbb, 0xf9, 0x40, 0xb4, 0x97, 0x80, 0x01, 0x1d);
DEFINE_GUID(IID_IEnumSpObjectTokens,      0x06B64F9E, 0x7FDA, 0x11D2, 0xb4, 0xf2, 0x00, 0xc0, 0x4f, 0x79, 0x73, 0x96);
DEFINE_GUID(IID_ISpEventSource,           0xBE7A9CCE, 0x5F9E, 0x11D2, 0x96, 0x0f, 0x00, 0xc0, 0x4f, 0x8e, 0xe6, 0x28);
DEFINE_GUID(IID_ISpNotifySource,          0x5EFF4AEF, 0x8487, 0x11D2, 0x96, 0x1c, 0x00, 0xc0, 0x4f, 0x8e, 0xe6, 0x28);
DEFINE_GUID(IID_ISpNotifySink,            0x259684DC, 0x37C3, 0x11D2, 0x96, 0x03, 0x00, 0xc0, 0x4f, 0x8e, 0xe6, 0x28);
DEFINE_GUID(IID_ISpDataKey,               0x14056581, 0xE16C, 0x11D2, 0xbb, 0x90, 0x00, 0xc0, 0x4f, 0x8e, 0xe6, 0xc0);
DEFINE_GUID(IID_ISpStreamFormat,          0xBED530BE, 0x2606, 0x4F4D, 0xa1, 0xc0, 0x54, 0xc5, 0xcd, 0xa5, 0x56, 0x6f);

