html
{
  background-color: white;
}

body
{
  margin-top: 1px;
  margin-right: 3px;
  margin-left:  2px;
  margin-bottom: 3px;
  background: white;
}

a
{
  text-decoration: none;
  color: darkblue;
}

a:hover
{
  text-decoration: underline;
}

.gdarticle
{
  margin-top: 0.1em;
  margin-bottom: 0.4em;
  padding: 5px;
  border: 1px solid #d0dde2;
  border-radius: 8px;
  background: white;
}

.gdactivearticle
{
  border: 1px solid #3399FF;
}

.gdarticleseparator + script + .gdactivearticle .gddictname
{
  border-top: 1px solid #3399FF;
}

.gdarticleseparator
{
  display: none;
  clear: both;
}

/* Hide the 'From ' string which precedes dictionary name in the heading */
.gdfromprefix
{
  display: none;
}

.gddictname
{
  display: -webkit-box; /* needed to be able to reorder elements, e.g. icon, dictionary name, collapse icon */
  font-size: 12px;
  font-weight: normal;
  float: right;
  border: 0px;
  border-top-right-radius: 8px;
  border-bottom-left-radius: 6px;
  margin: -6px;
  margin-bottom: 5px;
  margin-left: 2px;
  padding-right: 0.3em;
  color: #4480f8;
  background: #E0E8F0;
  user-select: none;
  cursor: default;
}

/* Actual text with the dictionary title */
.gddicttitle
{
  display:block;
  -webkit-box-ordinal-group: 1;
  -webkit-box-flex: 1;
}

.gdactivearticle .gddictname
{
  font-weight: normal;
  margin: -6px;
  margin-bottom: 5px;
  margin-left: 2px;

  float: right;
  border: 1px solid #3399FF;
  color: #0066cc;
  background: #deecf8;
}

.gdcollapsedarticle .gddictname
{
  opacity: 0.7;
}

/* Nice diagonal pattern for the collapsed article */
.gdcollapsedarticle
{
	background-image: -webkit-linear-gradient(left top, #ccc 0%, #ccc 25%, #bbb 25%, #bbb 50%, #ccc 50%, #ccc 75%, #bbb 75%);
	background-size: 50px 50px;
}

/* Move the collapse/expand buttons to the last, 3rd position */
.collapse_expand_area
{
  display: block;
  -webkit-box-ordinal-group: 3;
}

.gddicticon
{
  display: block;
  vertical-align: text-bottom;
  padding-right: -1em;
  padding-left: 0.3em;
  -webkit-box-ordinal-group: 2;
}

.gddicticon img {
    height: 1.3em;
}

.gdexpandicon, .gdexpandicon:hover, .gdcollapseicon, .gdcollapseicon:hover
{
    width:15px;
    background-position: center 35%;
    background-repeat:no-repeat;
    vertical-align: text-bottom;
    padding-left: -5px;
    margin-left: 0px;
}

.gdexpandicon {
    background-image: url('qrcx://localhost/icons/expand_article.png');
}

.gdexpandicon:hover {
    background-image: url('qrcx://localhost/icons/expand_article_hovered.png');
}

.gdcollapseicon {
    background-image: url('qrcx://localhost/icons/collapse_article.png');
}

.gdcollapseicon:hover {
    background-image: url('qrcx://localhost/icons/collapse_article_hovered.png');
}

/* The first headword in a (possibly) multi-headword DSL article */
.gdarticlebody > div:first-child .dsl_headwords,
{
  margin-top: -3px;
  display: inline-block;
}

.gdspellsuggestion
{
  /* Add some vertical space before the article */
  margin-top: 1.5em;
}

.dsl_ex, .dsl_ex .dsl_opt
{
  color: steelblue;
}

::selection {
  background:#cc0000;
  color:#fff;
}

code::selection {
  background: #333333;
}
