/* This file is (c) 2008-2018 Abs62
 * Part of GoldenDict. Licensed under GPLv3 or later, see the LICENSE file */

#include <QDropEvent>
#include "treeview.hh"

void TreeView::dropEvent( QDropEvent * event )
{
  QTreeView::dropEvent( event );
}
